/*  
 *
 * Copyright (C) 2000 Panagiotis Issaris <takis@lumumba.luc.ac.be> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * $Author: takis $
 * $Date: 2000/09/21 01:12:36 $
 * $Revision: 1.6 $
 *
 */

#include "gdk_pixbuf_ops.h"

static char matrix_sharpen[3][3] =
	{
		{-1,-1,-1},
		{-1,9,-1},
		{-1,-1,-1}
	};

static char matrix_edge_detect[3][3] =
	{
		{-1,-1,-1},
		{-1,8,-1},
		{-1,-1,-1}
	};

static float matrix_blur[3][3] =
	{
		{1.0/9.0,1.0/9.0,1.0/9.0},
		{1.0/9.0,1.0/9.0,1.0/9.0},
		{1.0/9.0,1.0/9.0,1.0/9.0}
	};



void gdk_pixbuf_putpix(GdkPixbuf *pb, int x, int y)
{
	int nc;
	int rs;
	unsigned char *p;
	guchar *q;

	rs = gdk_pixbuf_get_rowstride(pb);
	nc = gdk_pixbuf_get_n_channels(pb);
	p = gdk_pixbuf_get_pixels(pb);
	
	
	q = p + y*rs + (x*nc);
	*q = 0;
	*(q+1) = 0;
	*(q+2) = 0;
}

void gdk_pixbuf_op(GdkPixbuf *pb, byte_convertor red, byte_convertor green, byte_convertor blue)
{
  unsigned char *s;
  int x, y, c;
  int width, height, bps, nc, rs;
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  
  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {
	*s = red(*s);
	s++;
	*s = green(*s);
	s++;
	*s = blue(*s);
	s++;
    }
  }
  
}

void gdk_pixbuf_op1(GdkPixbuf *pb, byte_convertor component)
{
  unsigned char *s;
  int x, y, c;
  int width, height, bps, nc, rs;
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  
  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {
	    *s = component(*s);
	    s++;
	    *s = component(*s);
	    s++;
	    *s = component(*s);
	    s++;
    }
  }
  
}

unsigned char gdk_pixbuf_color_component_op_invert(unsigned char c)
{
	return ~c;
}

void gdk_pixbuf_invert(GdkPixbuf *pb)
{
 gdk_pixbuf_op1(pb, &gdk_pixbuf_color_component_op_invert); 
}

unsigned char gdk_pixbuf_color_component_op_keep ( unsigned char p)
{
	return p;
}

unsigned char gdk_pixbuf_color_component_op_zero ( unsigned char p)
{
	return 0;
}

void gdk_pixbuf_red(GdkPixbuf *pb)
{
	gdk_pixbuf_op (pb, &gdk_pixbuf_color_component_op_keep, 
		&gdk_pixbuf_color_component_op_zero, 
		&gdk_pixbuf_color_component_op_zero);
}

void gdk_pixbuf_green(GdkPixbuf *pb)
{
	gdk_pixbuf_op (pb, &gdk_pixbuf_color_component_op_zero, 
		&gdk_pixbuf_color_component_op_keep, 
		&gdk_pixbuf_color_component_op_zero);
}

void gdk_pixbuf_blue(GdkPixbuf *pb)
{
 	gdk_pixbuf_op (pb, &gdk_pixbuf_color_component_op_zero, 
		&gdk_pixbuf_color_component_op_zero, 
		&gdk_pixbuf_color_component_op_keep);
}
 

void gdk_pixbuf_grayscale(GdkPixbuf *pb)
{
  unsigned char *s;
  int x, y, c;
  unsigned char r,g,b;
  
  int width, height, bps, nc, rs;
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  
  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {

			r = *s;
			g = *(s+1);
			b = *(s+2);
			c = (r+g+b);
			c /= 3;
			*s = c;
			*(s+1) = c;
			*(s+2) = c;
			s+=3;
	}
  }
  
}

/*
 * This algorithm was taken from Gnoghurt.
 */
void gdk_pixbuf_emboss2(GdkPixbuf *src, GdkPixbuf **dst)
{
  unsigned char *s;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char *previous_input_row, *current_input_row, *next_input_row;
  unsigned char *output_row; 
  unsigned char *d;
  int width, height, bps, nc, rs;
  int xgrad, ygrad, val;
  static char *d_buffer=NULL;
  
  width = gdk_pixbuf_get_width (src);
  height = gdk_pixbuf_get_height (src);
  bps = gdk_pixbuf_get_bits_per_sample (src);
  nc = gdk_pixbuf_get_n_channels (src);
  rs = gdk_pixbuf_get_rowstride (src);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(width*height*3);

  s = gdk_pixbuf_get_pixels(src);
  d = d_buffer;
  
  for(y=1; y<height-1; y++)
  {
    previous_input_row = s + (y-1)*rs;
    current_input_row = s + (y)*rs;
    next_input_row = s + (y+1)*rs;
    output_row = d + y * rs;
   
    output_row[0] = output_row[1] = output_row[2] = 0; 
    
    for(x=1;x<width-1;x++)
    {
      xgrad = ((int)current_input_row[3] 
		      + (int)current_input_row[4] 
		      + (int)next_input_row[5]) 
	      - ((int)previous_input_row[-1] 
			      + (int)current_input_row[-2] 
			      + (int)next_input_row[-3]);
      ygrad = ((int)next_input_row[0] 
			  + (int)next_input_row[1] 
			  + (int)next_input_row[2]) 
	      - ((int)previous_input_row[0] 
			      + (int)previous_input_row[1] 
			      + (int)previous_input_row[2]);
      val = CLAMP ((xgrad + ygrad + 512) >> 3, 0, 255);

      *output_row++ = val;
      *output_row++ = val;
      *output_row++ = val;
      
      previous_input_row += 3;
      current_input_row += 3;
      next_input_row += 3;
    }
    output_row[0] = output_row[1] = output_row[2] = 0;
  }
  
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,width,height,width*3, NULL, NULL); 
}

void gdk_pixbuf_emboss(GdkPixbuf *pb)
{
  unsigned char *s;
  unsigned char *s2;
  int x, y, c;
  unsigned char r,g,b;
  GdkPixbuf *src;
  int width, height, bps, nc, rs;
  
  gdk_pixbuf_emboss2(pb, &src);
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  s2 = gdk_pixbuf_get_pixels(src);

  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {

	*s++ = *s2++;
	*s++ = *s2++;
	*s++ = *s2++;
    }
  }
}

void gdk_pixbuf_blur2(GdkPixbuf *src, GdkPixbuf **dst)
{
  unsigned char *s;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char *previous_input_row, *current_input_row, *next_input_row;
  unsigned char *output_row; 
  unsigned char *d;
  int width, height, bps, nc, rs;
  int xgrad, ygrad, val;
   int red, green, blue;
  static char *d_buffer=NULL;
  
  width = gdk_pixbuf_get_width (src);
  height = gdk_pixbuf_get_height (src);
  bps = gdk_pixbuf_get_bits_per_sample (src);
  nc = gdk_pixbuf_get_n_channels (src);
  rs = gdk_pixbuf_get_rowstride (src);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(width*height*3);

  s = gdk_pixbuf_get_pixels(src);
  d = d_buffer;
  
  for(y=1; y<height-1; y++)
  {
    previous_input_row = s + (y-1)*rs;
    current_input_row = s + (y)*rs;
    next_input_row = s + (y+1)*rs;
    output_row = d + y * rs;
   
    output_row[0] = output_row[1] = output_row[2] = 0; 
   /* Red Green Blue   Red Green Blue   Red Green Blue 
    * -3   -2    -1     0    1    2      3     4   5
    * */ 
    for(x=1;x<width-1;x++)
    {
      red = previous_input_row[-3] 
	      + previous_input_row[0] 
	      + previous_input_row[3]
	      + current_input_row[-3]
	     + current_input_row[0]
	    +current_input_row[3]
	   +next_input_row[-3]
	  +next_input_row[0]
	 +next_input_row[3]; 
      green = previous_input_row[-2] 
	      + previous_input_row[1] 
	      + previous_input_row[4]
	      + current_input_row[-2]
	     + current_input_row[1]
	    +current_input_row[4]
	   +next_input_row[-1]
	  +next_input_row[2]
	 +next_input_row[4]; 
       blue = previous_input_row[-1] 
	      + previous_input_row[2] 
	      + previous_input_row[5]
	      + current_input_row[-1]
	     + current_input_row[2]
	    +current_input_row[5]
	   +next_input_row[-1]
	  +next_input_row[2]
	 +next_input_row[5]; 
       
      red = CLAMP (red/9, 0, 255);
      green = CLAMP (green/9, 0, 255);
      blue = CLAMP (blue/9, 0, 255);

      *output_row++ = red;
      *output_row++ = green;
      *output_row++ = blue;
      
      previous_input_row += 3;
      current_input_row += 3;
      next_input_row += 3;
    }
    output_row[0] = output_row[1] = output_row[2] = 0;
  }
  
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,width,height,width*3, NULL, NULL); 
}

void gdk_pixbuf_blur(GdkPixbuf *pb)
{
  unsigned char *s;
  unsigned char *s2;
  int x, y, c;
  unsigned char r,g,b;
  GdkPixbuf *src;
  int width, height, bps, nc, rs;
  
  gdk_pixbuf_blur2(pb, &src);
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  s2 = gdk_pixbuf_get_pixels(src);

  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {

	*s++ = *s2++;
	*s++ = *s2++;
	*s++ = *s2++;
    }
  }
}

void gdk_pixbuf_apply_matrix2(GdkPixbuf *src, GdkPixbuf **dst, char matrix[3][3])
{
  unsigned char *s;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char *previous_input_row, *current_input_row, *next_input_row;
  unsigned char *output_row; 
  unsigned char *d;
  int width, height, bps, nc, rs;
  int xgrad, ygrad, val;
   int red, green, blue;
  static char *d_buffer=NULL;
  
  width = gdk_pixbuf_get_width (src);
  height = gdk_pixbuf_get_height (src);
  bps = gdk_pixbuf_get_bits_per_sample (src);
  nc = gdk_pixbuf_get_n_channels (src);
  rs = gdk_pixbuf_get_rowstride (src);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(width*height*3);

  s = gdk_pixbuf_get_pixels(src);
  d = d_buffer;
  
  for(y=1; y<height-1; y++)
  {
    previous_input_row = s + (y-1)*rs;
    current_input_row = s + (y)*rs;
    next_input_row = s + (y+1)*rs;
    output_row = d + y * rs;
   
    output_row[0] = output_row[1] = output_row[2] = 0; 
   /* Red Green Blue   Red Green Blue   Red Green Blue 
    * -3   -2    -1     0    1    2      3     4   5
    * */ 
    for(x=1;x<width-1;x++)
    {
      red = matrix[0][0] * previous_input_row[-3] 
	      + matrix[0][1] * previous_input_row[0] 
	      + matrix[0][2] * previous_input_row[3]
	      + matrix[1][0] * current_input_row[-3]
	     + matrix[1][1] * current_input_row[0]
	    +matrix[1][2] * current_input_row[3]
	   +matrix[2][0] * next_input_row[-3]
	  +matrix[2][1] * next_input_row[0]
	 +matrix[2][2] * next_input_row[3]; 
  
      green = matrix[0][0] * previous_input_row[-2] 
	      + matrix[0][1] * previous_input_row[1] 
	      + matrix[0][2] * previous_input_row[4]
	      + matrix[1][0] * current_input_row[-2]
	     + matrix[1][1] * current_input_row[1]
	    +matrix[1][2] * current_input_row[4]
	   +matrix[2][0] * next_input_row[-1]
	  +matrix[2][1] * next_input_row[2]
	 +matrix[2][2] * next_input_row[4]; 
  
      blue = matrix[0][0] * previous_input_row[-1] 
	      + matrix[0][1] * previous_input_row[2] 
	      + matrix[0][2] * previous_input_row[5]
	      + matrix[1][0] * current_input_row[-1]
	     + matrix[1][1] * current_input_row[2]
	    +matrix[1][2] * current_input_row[5]
	   +matrix[2][0] * next_input_row[-1]
	  +matrix[2][1] * next_input_row[2]
	 +matrix[2][2] * next_input_row[5]; 
       
      red = CLAMP (red, 0, 255);
      green = CLAMP (blue, 0, 255);
      blue = CLAMP (green, 0, 255);

      *output_row++ = red;
      *output_row++ = green;
      *output_row++ = blue;
      
      previous_input_row += 3;
      current_input_row += 3;
      next_input_row += 3;
    }
    output_row[0] = output_row[1] = output_row[2] = 0;
  }
  
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,width,height,width*3, NULL, NULL); 
}

void gdk_pixbuf_apply_matrix(GdkPixbuf *pb, char*matrix)
{
  unsigned char *s;
  unsigned char *s2;
  int x, y, c;
  unsigned char r,g,b;
  GdkPixbuf *src;
  int width, height, bps, nc, rs;
  
  gdk_pixbuf_apply_matrix2(pb, &src, matrix);
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  s2 = gdk_pixbuf_get_pixels(src);

  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {

	*s++ = *s2++;
	*s++ = *s2++;
	*s++ = *s2++;
    }
  }
}

void gdk_pixbuf_apply_matrixf2(GdkPixbuf *src, GdkPixbuf **dst, float matrix[3][3])
{
  unsigned char *s;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char *previous_input_row, *current_input_row, *next_input_row;
  unsigned char *output_row; 
  unsigned char *d;
  int width, height, bps, nc, rs;
  int xgrad, ygrad, val;
   int red, green, blue;
  static char *d_buffer=NULL;
  float fred, fgreen, fblue;

  width = gdk_pixbuf_get_width (src);
  height = gdk_pixbuf_get_height (src);
  bps = gdk_pixbuf_get_bits_per_sample (src);
  nc = gdk_pixbuf_get_n_channels (src);
  rs = gdk_pixbuf_get_rowstride (src);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(width*height*3);

  s = gdk_pixbuf_get_pixels(src);
  d = d_buffer;
  
  for(y=1; y<height-1; y++)
  {
    previous_input_row = s + (y-1)*rs;
    current_input_row = s + (y)*rs;
    next_input_row = s + (y+1)*rs;
    output_row = d + y * rs;
   
    output_row[0] = output_row[1] = output_row[2] = 0; 
   /* Red Green Blue   Red Green Blue   Red Green Blue 
    * -3   -2    -1     0    1    2      3     4   5
    * */ 
    for(x=1;x<width-1;x++)
    {
      fred = matrix[0][0] * previous_input_row[-3] 
	      + matrix[0][1] * previous_input_row[0] 
	      + matrix[0][2] * previous_input_row[3]
	      + matrix[1][0] * current_input_row[-3]
	     + matrix[1][1] * current_input_row[0]
	    +matrix[1][2] * current_input_row[3]
	   +matrix[2][0] * next_input_row[-3]
	  +matrix[2][1] * next_input_row[0]
	 +matrix[2][2] * next_input_row[3]; 
      fgreen = matrix[0][0] * previous_input_row[-2] 
	      + matrix[0][1] * previous_input_row[1] 
	      + matrix[0][2] * previous_input_row[4]
	      + matrix[1][0] * current_input_row[-2]
	     + matrix[1][1] * current_input_row[1]
	    +matrix[1][2] * current_input_row[4]
	   +matrix[2][0] * next_input_row[-1]
	  +matrix[2][1] * next_input_row[2]
	 +matrix[2][2] * next_input_row[4]; 
       fblue = matrix[0][0] * previous_input_row[-1] 
	      + matrix[0][1] * previous_input_row[2] 
	      + matrix[0][2] * previous_input_row[5]
	      + matrix[1][0] * current_input_row[-1]
	     + matrix[1][1] * current_input_row[2]
	    +matrix[1][2] * current_input_row[5]
	   +matrix[2][0] * next_input_row[-1]
	  +matrix[2][1] * next_input_row[2]
	 +matrix[2][2] * next_input_row[5]; 
       
      red = CLAMP (fred, 0, 255);
      green = CLAMP (fgreen, 0, 255);
      blue = CLAMP (fblue, 0, 255);

      *output_row++ = red;
      *output_row++ = green;
      *output_row++ = blue;
      
      previous_input_row += 3;
      current_input_row += 3;
      next_input_row += 3;
    }
    output_row[0] = output_row[1] = output_row[2] = 0;
  }
  
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,width,height,width*3, NULL, NULL); 
}

void gdk_pixbuf_apply_matrixf(GdkPixbuf *pb, float*matrix)
{
  unsigned char *s;
  unsigned char *s2;
  int x, y, c;
  unsigned char r,g,b;
  GdkPixbuf *src;
  int width, height, bps, nc, rs;
  
  gdk_pixbuf_apply_matrixf2(pb, &src, matrix);
  
  width = gdk_pixbuf_get_width (pb);
  height = gdk_pixbuf_get_height (pb);
  bps = gdk_pixbuf_get_bits_per_sample (pb);
  nc = gdk_pixbuf_get_n_channels (pb);
  rs = gdk_pixbuf_get_rowstride (pb);

  s = gdk_pixbuf_get_pixels(pb);
  s2 = gdk_pixbuf_get_pixels(src);

  for(y=0; y<height; y++)
  {
    for(x=0;x<width;x++)
    {

	*s++ = *s2++;
	*s++ = *s2++;
	*s++ = *s2++;
    }
  }
}

void gdk_pixbuf_sharpen(GdkPixbuf *pb)
{
  gdk_pixbuf_apply_matrix(pb, matrix_sharpen);
}

void gdk_pixbuf_edge_detect(GdkPixbuf *pb)
{
  gdk_pixbuf_apply_matrix(pb, matrix_edge_detect);
}

void gdk_pixbuf_blurm(GdkPixbuf *pb)
{
  gdk_pixbuf_apply_matrixf(pb, matrix_blur);
}

void gdk_pixbuf_value_color(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst)
{
		GdkPixbuf *value,*color,*tmp;
		value = gdk_pixbuf_copy(src1);
		gdk_pixbuf_grayscale(value);
		
		tmp = gdk_pixbuf_copy(src2);
		gdk_pixbuf_grayscale(tmp);
		gdk_pixbuf_sub(src2,tmp,&color);

		gdk_pixbuf_add(value,color,dst);
		gdk_pixbuf_unref(value);
		gdk_pixbuf_unref(color);
		gdk_pixbuf_unref(tmp);
}

void gdk_pixbuf_blend(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst, unsigned char a)
{
  unsigned char *s1, *s2, *d;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char r1,g1,b1;
  unsigned char r2,g2,b2;
  int ir,ig,ib;
  int argb;
  int src1_width, src1_height, src1_bps, src1_nc, src1_rs;
//static char d_buffer[300*300*3];
  static char *d_buffer=NULL;
  
  src1_width = gdk_pixbuf_get_width (src1);
  src1_height = gdk_pixbuf_get_height (src1);
  src1_bps = gdk_pixbuf_get_bits_per_sample (src1);
  src1_nc = gdk_pixbuf_get_n_channels (src1);
  src1_rs = gdk_pixbuf_get_rowstride (src1);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(src1_width*src1_height*3);

  s1 = gdk_pixbuf_get_pixels(src1);
  s2 = gdk_pixbuf_get_pixels(src2);
  d = d_buffer;
#ifndef USE_MMX 
  for(y=0; y<src1_height; y++)
  {
    for(x=0;x<src1_width;x++)
    {
	  r1 = *s1;
	  r2 = *s2;
      ir = (r1 * a + (255 - a) * r2) / 255;
	  r = (unsigned char) ir;
	  *d = r;
	  s1++;
	  s2++;
	  d++;
	  
	  g1 = *s1;
	  g2 = *s2;
      ig = ( (g1 * a + (255 - a) * g2) ) / 255;
	  g = (unsigned char) ig;
	  *d = g;
	  s1++;
	  s2++;
	  d++;
	  
	  b1 = *s1;
	  b2 = *s2;
      ib = (( b1 * a + (255 - a) * b2) ) / 255;
	  b = (unsigned char) ib;
	  *d = b;
	  s1++;
	  s2++;
	  d++;	  
	}
  //  s1 += (src1_rs - 300);
  //  s2 += (src1_rs - 300);
  }
#else
  blendd(s1, s2, d, src1_width, src1_height); 
#endif
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,src1_width,src1_height,src1_width*3, NULL, NULL); 
}

unsigned char mul_pixel(unsigned char a, unsigned char b)
{
	int p = a*b;
	if(p>255)
		return 255;
	else
		return p;
}
unsigned char div_pixel(unsigned char a, unsigned char b)
{
	if(b!=0)	
		return a/b;
	else
		0;
}

unsigned char sub_pixel(unsigned char a, unsigned char b)
{
	int s = a - b;
	if(s<0)
		return 0;
	else
		return s;
}

unsigned char add_pixel(unsigned char a, unsigned char b)
{
	int s = a+b;
	if(s>255)
		return 255;
	else
		return s;
}

void gdk_pixbuf_add(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst)
{
	gdk_pixbuf_op2(src1,src2,dst,&add_pixel,&add_pixel,&add_pixel);
}

void gdk_pixbuf_sub(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst)
{
	gdk_pixbuf_op2(src1,src2,dst,&sub_pixel,&sub_pixel,&sub_pixel);
}


void gdk_pixbuf_mul(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst)
{
	gdk_pixbuf_op2(src1,src2,dst,&mul_pixel,&mul_pixel,&mul_pixel);
}

void gdk_pixbuf_div(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst)
{
	gdk_pixbuf_op2(src1,src2,dst,&div_pixel,&div_pixel,&div_pixel);
}

void gdk_pixbuf_op2(GdkPixbuf *src1, GdkPixbuf *src2, GdkPixbuf **dst,
		 byte_convertor2 red, byte_convertor2 green, byte_convertor2 blue )
{
  unsigned char *s1, *s2, *d;
  int x, y, c;
  unsigned char r,g,b;
  unsigned char r1,g1,b1;
  unsigned char r2,g2,b2;
  int ir,ig,ib;
  int argb;
  int src1_width, src1_height, src1_bps, src1_nc, src1_rs;
//static char d_buffer[300*300*3];
  static char *d_buffer=NULL;
  
  src1_width = gdk_pixbuf_get_width (src1);
  src1_height = gdk_pixbuf_get_height (src1);
  src1_bps = gdk_pixbuf_get_bits_per_sample (src1);
  src1_nc = gdk_pixbuf_get_n_channels (src1);
  src1_rs = gdk_pixbuf_get_rowstride (src1);
  
  if(d_buffer==NULL)
  	d_buffer = malloc(src1_width*src1_height*3);

  s1 = gdk_pixbuf_get_pixels(src1);
  s2 = gdk_pixbuf_get_pixels(src2);
  d = d_buffer;
  for(y=0; y<src1_height; y++)
  {
    for(x=0;x<src1_width;x++)
    {
	*d = red(*s1, *s2);
	  s1++;
	  s2++;
	  d++;
	  
	 *d = green(*s1, *s2);
	  s1++;
	  s2++;
	  d++;
	  
	  *d = blue(*s1, *s2);
	  s1++;
	  s2++;
	  d++;	  
	}
  }
  
    *dst = gdk_pixbuf_new_from_data (d_buffer,
		  GDK_COLORSPACE_RGB,FALSE,
		  8,src1_width,src1_height,src1_width*3, NULL, NULL); 
}

