/*  
 *
 * Copyright (C) 2000 Panagiotis Issaris <takis@lumumba.luc.ac.be> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * $Author: takis $
 * $Date: 2000/09/23 13:40:15 $
 * $Revision: 1.18 $
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <math.h>
#include <libmpeg3/libmpeg3.h>
#include <gnome.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <pthread.h>

#include "MPEG_Movie.h"
#include "interface.h"
#include "callbacks.h"
#include "support.h"

extern GtkWidget* drawing_area;
extern GtkWidget *application;
extern GtkAdjustment *adjustment;
extern GList *effects_list;
fb_t *fb;

GdkPixbuf *frame;
GdkPixbuf *framez;
GdkEvent *event;
GdkEventType type;
GdkEvent e;

MPEG_Movie *mm;

guchar *p;
char *filename;
int mmc;
int play; 
int i;
int error_code;
int drawn;
int zoom;
int zoomfactor;
int bps,nc,rs;
int width, height;
int offset_x, offset_y;
int window_width, window_height;
guint timer_id;

pthread_t th_p;
pthread_mutex_t lock;
pthread_cond_t cond;

void draw_frame(GtkWidget *widget)
{
	int j;
	
	pthread_mutex_lock(&lock);
	
	if(fb->nrframes>0)
	{
		//printf("[%d]",fb->nrframes);
			
			/*
		width = gdk_pixbuf_get_width (fb->frames[0]);
		height = gdk_pixbuf_get_height (fb->frames[0]);
		gdk_pixbuf_render_to_drawable (fb->frames[0], 
			widget->window,
			widget->style->white_gc, 
			0, 0,
			0, 0,
			width,height,
			GDK_RGB_DITHER_NORMAL,
			0,0);*/
		draw_pixbuf(widget, fb->frames[0]);
		gdk_pixbuf_unref(fb->frames[0]);
		drawn = TRUE;
		fb->nrframes--;
		for(j=0;j<fb->nrframes;j++)
		{
				fb->frames[j]=fb->frames[j+1];
		}
	}
	pthread_mutex_unlock(&lock);
	
}

static gint
draw_pixbuf (GtkWidget *widget, GdkPixbuf *frame)
{
	guchar *pixels;
	int rowstride;
	
	width = gdk_pixbuf_get_width (frame);
	height = gdk_pixbuf_get_height (frame);
	switch(zoom)
	{
		case ZOOM_NOT:
			gdk_pixbuf_render_to_drawable (frame, 
				widget->window,
				widget->style->white_gc, 
				0, 0,
				0, 0,
				width,height,
				GDK_RGB_DITHER_NORMAL,
				0,0);
		break;
		case ZOOM_IN:
		
#ifdef ZOOM_SIMPLE
				framez = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
						FALSE, 8,
						width*zoomfactor,
						height*zoomfactor);
				framez = gdk_pixbuf_scale_simple (frame, 
					width*zoomfactor,height*zoomfactor, GDK_INTERP_BILINEAR);
#else
		//		printf("old framez: %p",framez);
				framez = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
						FALSE, 8,
						width*zoomfactor,
						height*zoomfactor);
		//		printf("new framez: %p",framez);
				g_assert(framez!=NULL);
				gdk_pixbuf_scale(frame, framez,
						0,0,
						width*zoomfactor,height*zoomfactor,
						-offset_x,-offset_y,
						zoomfactor,zoomfactor,
						GDK_INTERP_BILINEAR);
#endif
				gdk_pixbuf_render_to_drawable (framez, 
					widget->window,
					widget->style->white_gc, 
					0, 0,
					0, 0,
					width*zoomfactor,height*zoomfactor,
					GDK_RGB_DITHER_NORMAL,
					0,0);
				gdk_pixbuf_unref(framez);
				break;
			case ZOOM_OUT:
				framez = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
						FALSE, 8,
						width/zoomfactor+1,
						height/zoomfactor+1);
				framez = gdk_pixbuf_scale_simple (frame, 
					width/(zoomfactor),height/zoomfactor, GDK_INTERP_BILINEAR);
				gdk_pixbuf_render_to_drawable (framez, 
					widget->window,
					widget->style->white_gc, 
					offset_x, offset_y,
					0,0,
					width/(zoomfactor),height/(zoomfactor),
					GDK_RGB_DITHER_NORMAL,
					0,0);
				gdk_pixbuf_unref(framez);
/*				framez = gdk_pixbuf_scale_simple (frame, 
					width/(zoomfactor),height/zoomfactor, GDK_INTERP_BILINEAR);
				gdk_pixbuf_render_to_drawable (framez, 
					widget->window,
					widget->style->white_gc, 
					offset_x, offset_y,
					0,0,
					width/(zoomfactor),height/(zoomfactor),
					GDK_RGB_DITHER_NORMAL,
					0,0);
				gdk_pixbuf_unref(framez);
*/				break;
			}
		//gdk_pixbuf_unref(frame);
		drawn = TRUE;
	return TRUE;
}

void init()
{
	frame=NULL;
	zoomfactor = 1;
	zoom = 0;
	offset_x = 0;
	offset_y = 0;

	fb->maxframes=4;
	fb->nrframes=0;

	width = 320;
	height = 240;
	window_width = 320;
	window_height = 240;

	gtk_widget_set_usize (drawing_area, width, height);
	gtk_widget_set_usize (application, window_width, window_height +WINDOW_EXTRA);
	adjustment->upper = 1;
}

void open_file(char *movie_filename)
{
	init();
	mm = MPEG_Movie_load (movie_filename);	

	width = mm->width;
	height = mm->height;
	window_width = width;
	window_height = height;

	gtk_widget_set_usize (drawing_area, width, height);
	gtk_widget_set_usize (application, window_width, window_height +WINDOW_EXTRA);
	adjustment->upper = mm->frames;
	if(timer_id==0)
		timer_id = gtk_timeout_add (1000/mm->framerate,on_timeout, (gpointer)drawing_area);
}

int
main (int argc, char *argv[])
{
	char *movie_filename;
	char *movie2_filename;
	int width, height;
	
#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif
	
	gnome_init("GdkPixbuf Player Takis", VERSION, argc, argv);
	gdk_rgb_init ();

	fb = g_malloc0(sizeof(fb_t));

	timer_id = 0;

	effects_list = NULL;

	application = create_application();
	
	init();	
	
	if(argc<=1)
	{
		printf(_("Usage: %s movie1.mpg [movie2.mpg] [scalingmode] [backgroundimage.ext] \n"), argv[0]);	
	//	exit(0);
		
	}
	else
	{
		movie_filename = argv[1];
	
		if(argc>2)
			movie2_filename = argv[2];
		else
			movie2_filename = movie_filename;
		open_file(movie_filename);	

	}
	
	
	gtk_widget_show_all (application);
	
	pthread_mutex_init(&lock, NULL);
	if(error_code = pthread_create(&th_p, NULL, play_thread, (gpointer)drawing_area))
		printf(_("Creation of thread failed with errorcode %d\n"),error_code);
	else
		printf(_("Thread succesfully created.\n"));
//pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
//	gtk_idle_add (on_idle, (gpointer)da);
	//gtk_timeout_add (TIMEOUT_VALUE,on_timeout, (gpointer)da);
	gtk_main ();
	pthread_mutex_destroy(&lock);
	pthread_join(&th_p, NULL);
	return 0;
}


